var org_basic_concrete = require('./org_basic_concrete.js');
var org_basic_webacid = require('./org_basic_webacid.js');
var org_basic_core = require('./org_basic_core.js');
var org_basic_http = require('./org_basic_http.js');

/* ------------------------------------------------------------------------ */

var issuesCount = 0;

/* ------------------------------------------------------------------------ */

function resetIssuesCount() {
	issuesCount = 0;
	
	setBadge('');
}

function increaseIssuesCount() {
	setBadge(++issuesCount);
}

/* ------------------------------------------------------------------------ */

var globalEngine = null;

/* ------------------------------------------------------------------------ */

function handleWorkerMessage(message) {
	if (('startTest' in message)) {
		resetIssuesCount();
		
		var workspace = new org_basic_concrete.TestWorkspace();
		var client = new org_basic_concrete.TestClient();
		var scheduler = new org_basic_webacid.Scheduler(workspace, client, 4);
		var reporter = new org_basic_concrete.TestReporter(workspace);
		var progressor = new org_basic_webacid.Progressor(scheduler, 10);
		var engine = new org_basic_webacid.BasicEngine(workspace, scheduler, reporter, progressor);
		
		setProperty('issues', []);
		
		reporter.registerObserver({
			handleReportedStatement: function (statement) {
				org_basic_core.LogUtils.recordMessage('issue: ' + statement.getSummary());
				
				var issue = statement.getIssue();
				var transaction = issue.getTransaction();
				var url = issue.getUrl();
				
				if (!url) {
					url = transaction.getRequest().getUrl().make();
				}
				
				var issueItem = {title: statement.getTitle(), summary: statement.getSummary(), exact: statement.getExact(), description: statement.getDescription(), details: statement.getDetails()};
				
				sendMessage({issue: issueItem});
				
				increaseIssuesCount();
				
				var issues = getProperty('issues');
				
				issues.push(issueItem);
				
				setProperty('issues', issues);
			}
		});
		
		progressor.registerObserver({
			handleProgressNotification: function (percentage, step, steps, status) {
				org_basic_core.LogUtils.recordMessage('progress: ' + status);
				
				sendMessage({progress: {percentage: percentage, step: step, steps: steps, status: status}});
				
				setProperty('percentage', percentage);
				setProperty('step', step);
				setProperty('step', steps);
				setProperty('status', status);
				
				if (percentage == 100) {
					globalEngine = null;
					
					setProperty('state', 'stopped');
					sendMessage({stopTest: {}});
				}
			}
		});
		
		engine.launchUrl(org_basic_http.Url.parse(message.startTest.url));
		
		setProperty('current', message.startTest.url);
		setProperty('state', 'running');
		
		globalEngine = engine;
	} else
	if ('stopTest' in message) {
		if (globalEngine != null) {
			globalEngine.pause();
		}
		
		setProperty('state', 'stopped');
		
		globalEngine = null;
	} else
	if ('pauseTest' in message) {
		if (globalEngine != null) {
			globalEngine.pause();
		}
		
		setProperty('state', 'resting');
	} else
	if ('resumeTest' in message) {
		if (globalEngine != null) {
			globalEngine.resume();
		}
		
		setProperty('state', 'running');
	} else
	if ('resetTest' in message) {
		resetIssuesCount();
		
		setProperty('issues', null);
		setProperty('current', null);
		setProperty('state', null);
		setProperty('percentage', null);
		setProperty('step', null);
		setProperty('steps', null);
		setProperty('status', null);
	}
}
